@@ -1,4 +1,4 @@
-_CONDA_EXE="$(cygpath 'C:\bld\conda_1523624483710\_b_env\Scripts\conda.exe')"
+_CONDA_EXE="$(cygpath 'C:\bld\conda_1525135410775\_h_env\Scripts\conda.exe')"
 _conda_set_vars() {
     # set _CONDA_SHELL_FLAVOR
     if [ -n "${BASH_VERSION:+x}" ]; then
