@@ -1,44 +1,49 @@
 # -*- coding: utf-8 -*-
 from __future__ import absolute_import, division, print_function, unicode_literals
 
-from errno import EACCES, EEXIST, EPERM
+from errno import EACCES, ELOOP, EPERM
 from io import open
-import json
 from logging import getLogger
 import os
-from os import X_OK, access, makedirs
-from os.path import basename, isdir, isfile, join, lexists
-from shutil import copy as shutil_copy, copystat
+from os import X_OK, access
+from os.path import basename, dirname, isdir, isfile, join, splitext
+from shutil import copyfileobj, copystat
 import sys
 import tarfile
 
+from . import mkdir_p
 from .delete import rm_rf
-from .link import islink, link, readlink, symlink
+from .link import islink, lexists, link, readlink, symlink
 from .permissions import make_executable
-from .read import get_json_content
 from .update import touch
 from ..subprocess import subprocess_call
 from ... import CondaError
-from ..._vendor.auxlib.entity import EntityEncoder
 from ..._vendor.auxlib.ish import dals
+from ...base.constants import PACKAGE_CACHE_MAGIC_FILE
 from ...base.context import context
 from ...common.compat import ensure_binary, on_win
-from ...common.path import win_path_ok
-from ...core.portability import replace_long_shebang
-from ...exceptions import BasicClobberError, CondaOSError, maybe_raise
-from ...models.dist import Dist
+from ...common.path import ensure_pad, expand, win_path_double_escape, win_path_ok
+from ...common.serialize import json_dump
+from ...exceptions import (BasicClobberError, CaseInsensitiveFileSystemError, CondaOSError,
+                           maybe_raise)
 from ...models.enums import FileMode, LinkType
 
 log = getLogger(__name__)
-stdoutlog = getLogger('stdoutlog')
+stdoutlog = getLogger('conda.stdoutlog')
 
+# in __init__.py to help with circular imports
+mkdir_p = mkdir_p
 
 python_entry_point_template = dals("""
 # -*- coding: utf-8 -*-
+import re
+import sys
+
+from %(module)s import %(import_name)s
+
 if __name__ == '__main__':
-    from sys import exit
-    from %(module)s import %(func)s
-    exit(%(func)s())
+    sys.argv[0] = re.sub(r'(-script\.pyw?|\.exe)?$', '', sys.argv[0])
+    sys.exit(%(func)s())
 """)
 
 application_entry_point_template = dals("""
@@ -46,19 +51,21 @@ application_entry_point_template = dals("""
 if __name__ == '__main__':
     import os
     import sys
-    os.execv(%(source_full_path)s, sys.argv)
+    args = ["%(source_full_path)s"]
+    if len(sys.argv) > 1:
+        args += sys.argv[1:]
+    os.execv(args[0], args)
 """)
 
 
 def write_as_json_to_file(file_path, obj):
     log.trace("writing json to file %s", file_path)
     with open(file_path, str('wb')) as fo:
-        json_str = json.dumps(obj, indent=2, sort_keys=True, separators=(',', ': '),
-                              cls=EntityEncoder)
+        json_str = json_dump(obj)
         fo.write(ensure_binary(json_str))
 
 
-def create_unix_python_entry_point(target_full_path, python_full_path, module, func):
+def create_python_entry_point(target_full_path, python_full_path, module, func):
     if lexists(target_full_path):
         maybe_raise(BasicClobberError(
             source_path=None,
@@ -66,22 +73,40 @@ def create_unix_python_entry_point(target_full_path, python_full_path, module, f
             context=context,
         ), context)
 
-    pyscript = python_entry_point_template % {'module': module, 'func': func}
-    with open(target_full_path, str('w')) as fo:
+    import_name = func.split('.')[0]
+    pyscript = python_entry_point_template % {
+        'module': module,
+        'func': func,
+        'import_name': import_name,
+    }
+
+    if python_full_path is not None:
         shebang = '#!%s\n' % python_full_path
         if hasattr(shebang, 'encode'):
             shebang = shebang.encode()
+
+        from ...core.portability import replace_long_shebang  # TODO: must be in wrong spot
         shebang = replace_long_shebang(FileMode.text, shebang)
+
         if hasattr(shebang, 'decode'):
             shebang = shebang.decode()
-        fo.write(shebang)
+    else:
+        shebang = None
+
+    with open(target_full_path, str('w')) as fo:
+        if shebang is not None:
+            fo.write(shebang)
         fo.write(pyscript)
-    make_executable(target_full_path)
+
+    if shebang is not None:
+        make_executable(target_full_path)
 
     return target_full_path
 
 
-def create_windows_python_entry_point(target_full_path, module, func):
+def create_application_entry_point(source_full_path, target_full_path, python_full_path):
+    # source_full_path: where the entry point file points to
+    # target_full_path: the location of the new entry point file being created
     if lexists(target_full_path):
         maybe_raise(BasicClobberError(
             source_path=None,
@@ -89,14 +114,20 @@ def create_windows_python_entry_point(target_full_path, module, func):
             context=context,
         ), context)
 
-    pyscript = python_entry_point_template % {'module': module, 'func': func}
-    with open(target_full_path, str('w')) as fo:
-        fo.write(pyscript)
-
-    return target_full_path
+    entry_point = application_entry_point_template % {
+        "source_full_path": win_path_double_escape(source_full_path),
+    }
+    if not isdir(dirname(target_full_path)):
+        mkdir_p(dirname(target_full_path))
+    with open(target_full_path, str("w")) as fo:
+        if ' ' in python_full_path:
+            python_full_path = ensure_pad(python_full_path, '"')
+        fo.write('#!%s\n' % python_full_path)
+        fo.write(entry_point)
+    make_executable(target_full_path)
 
 
-def extract_tarball(tarball_full_path, destination_directory=None):
+def extract_tarball(tarball_full_path, destination_directory=None, progress_update_callback=None):
     if destination_directory is None:
         destination_directory = tarball_full_path[:-8]
     log.debug("extracting %s\n  to %s", tarball_full_path, destination_directory)
@@ -104,7 +135,27 @@ def extract_tarball(tarball_full_path, destination_directory=None):
     assert not lexists(destination_directory), destination_directory
 
     with tarfile.open(tarball_full_path) as t:
-        t.extractall(path=destination_directory)
+        members = t.getmembers()
+        num_members = len(members)
+
+        def members_with_progress():
+            for q, member in enumerate(members):
+                if progress_update_callback:
+                    progress_update_callback(q / num_members)
+                yield member
+
+        try:
+            t.extractall(path=destination_directory, members=members_with_progress())
+        except EnvironmentError as e:
+            if e.errno == ELOOP:
+                raise CaseInsensitiveFileSystemError(
+                    package_location=tarball_full_path,
+                    extract_location=destination_directory,
+                    caused_by=e,
+                )
+            else:
+                raise
+
     if sys.platform.startswith('linux') and os.getuid() == 0:
         # When extracting as root, tarfile will by restore ownership
         # of extracted files.  However, we want root to be the owner
@@ -115,22 +166,6 @@ def extract_tarball(tarball_full_path, destination_directory=None):
                 os.lchown(p, 0, 0)
 
 
-def write_linked_package_record(prefix, record):
-    # write into <env>/conda-meta/<dist>.json
-    meta_dir = join(prefix, 'conda-meta')
-    if not isdir(meta_dir):
-        makedirs(meta_dir)
-    dist = Dist(record)
-    conda_meta_full_path = join(meta_dir, dist.to_filename('.json'))
-    if lexists(conda_meta_full_path):
-        maybe_raise(BasicClobberError(
-            source_path=None,
-            target_path=conda_meta_full_path,
-            context=context,
-        ), context)
-    write_as_json_to_file(conda_meta_full_path, record)
-
-
 def make_menu(prefix, file_path, remove=False):
     """
     Create cross-platform menu items (e.g. Windows Start Menu)
@@ -151,18 +186,6 @@ def make_menu(prefix, file_path, remove=False):
         stdoutlog.error("menuinst Exception", exc_info=True)
 
 
-def mkdir_p(path):
-    try:
-        log.trace('making directory %s', path)
-        if path:
-            makedirs(path)
-    except OSError as e:
-        if e.errno == EEXIST and isdir(path):
-            return path
-        else:
-            raise
-
-
 def create_hard_link_or_copy(src, dst):
     if islink(src):
         message = dals("""
@@ -202,6 +225,17 @@ def _do_softlink(src, dst):
         symlink(src, dst)
 
 
+def create_fake_executable_softlink(src, dst):
+    assert on_win
+    src_root, _ = splitext(src)
+    # TODO: this open will clobber, consider raising
+    with open(dst, 'w') as f:
+        f.write("@echo off\n"
+                "call \"%s\" %%*\n"
+                "" % src_root)
+    return dst
+
+
 def copy(src, dst):
     # on unix, make sure relative symlinks stay symlinks
     if not on_win and islink(src):
@@ -216,7 +250,17 @@ def copy(src, dst):
 
 def _do_copy(src, dst):
     log.trace("copying %s => %s", src, dst)
-    shutil_copy(src, dst)
+    # src and dst are always files. So we can bypass some checks that shutil.copy does.
+    # Also shutil.copy calls shutil.copymode, which we can skip because we are explicitly
+    # calling copystat.
+
+    # Same size as used by Linux cp command (has performance advantage).
+    # Python's default is 16k.
+    buffer_size = 4194304  # 4 * 1024 * 1024  == 4 MB
+    with open(src, 'rb') as fsrc:
+        with open(dst, 'wb') as fdst:
+            copyfileobj(fsrc, fdst, buffer_size)
+
     try:
         copystat(src, dst)
     except (IOError, OSError) as e:  # pragma: no cover
@@ -254,6 +298,7 @@ def create_link(src, dst, link_type=LinkType.hardlink, force=False):
             log.trace("hard linking %s => %s", src, dst)
             link(src, dst)
         except (IOError, OSError) as e:
+            log.debug("%r", e)
             log.debug("hard-link failed. falling back to copy\n"
                       "  error: %r\n"
                       "  src: %s\n"
@@ -273,58 +318,35 @@ def compile_pyc(python_exe_full_path, py_full_path, pyc_full_path):
 
     command = '"%s" -Wi -m py_compile "%s"' % (python_exe_full_path, py_full_path)
     log.trace(command)
-    subprocess_call(command, raise_on_error=False)
+    result = subprocess_call(command, raise_on_error=False)
 
     if not isfile(pyc_full_path):
         message = dals("""
         pyc file failed to compile successfully
-          python_exe_full_path: %()s\n
-          py_full_path: %()s\n
-          pyc_full_path: %()s\n
+          python_exe_full_path: %s
+          py_full_path: %s
+          pyc_full_path: %s
+          compile rc: %s
+          compile stdout: %s
+          compile stderr: %s
         """)
-        log.info(message, python_exe_full_path, py_full_path, pyc_full_path)
+        log.info(message, python_exe_full_path, py_full_path, pyc_full_path,
+                 result.rc, result.stdout, result.stderr)
         return None
 
     return pyc_full_path
 
 
-def create_private_envs_meta(pkg, root_prefix, private_env_prefix):
-    # type: (str, str, str) -> ()
-    path_to_conda_meta = join(root_prefix, "conda-meta")
-
-    if not isdir(path_to_conda_meta):
-        mkdir_p(path_to_conda_meta)
-
-    private_envs_json = get_json_content(context.private_envs_json_path)
-    private_envs_json[pkg] = private_env_prefix
-    write_as_json_to_file(context.private_envs_json_path, private_envs_json)
-
-
-def create_private_pkg_entry_point(source_full_path, target_full_path, python_full_path):
-    if lexists(target_full_path):
-        maybe_raise(BasicClobberError(
-            source_path=None,
-            target_path=target_full_path,
-            context=context,
-        ), context)
-
-    entry_point = application_entry_point_template % {"source_full_path": source_full_path}
-    with open(target_full_path, str("w")) as fo:
-        fo.write('#!%s\n' % python_full_path)
-        fo.write(entry_point)
-    make_executable(target_full_path)
-
-
 def create_package_cache_directory(pkgs_dir):
     # returns False if package cache directory cannot be created
     try:
         log.trace("creating package cache directory '%s'", pkgs_dir)
-        mkdir_p(pkgs_dir)
-        touch(join(pkgs_dir, 'urls'))
-        touch(join(pkgs_dir, 'urls.txt'))
+        sudo_safe = expand(pkgs_dir).startswith(expand('~'))
+        touch(join(pkgs_dir, PACKAGE_CACHE_MAGIC_FILE), mkdir=True, sudo_safe=sudo_safe)
+        touch(join(pkgs_dir, 'urls'), sudo_safe=sudo_safe)
     except (IOError, OSError) as e:
         if e.errno in (EACCES, EPERM):
-            log.trace("Cannot create package cache directory '%s'", pkgs_dir)
+            log.trace("cannot create package cache directory '%s'", pkgs_dir)
             return False
         else:
             raise
